using System.Collections.Generic;
using System.Linq;
using MacGruber;
using SimpleJSON;
using UnityEngine;
using UnityEngine.Serialization;
using UnityEngine.UI;

namespace CheesyFX
{
    public abstract class Fuckable : MonoBehaviour
    {
        public new string name;
        public int type;
        public virtual bool isPenetrated { get; set; }
        protected static List<object> UIElements = new List<object>();
        public JSONStorableString info = new JSONStorableString("info", "Idle");
        public JSONStorableFloat depth = new JSONStorableFloat("Depth", 0f, 0f, .4f, true, false);
        public JSONStorableFloat speed = new JSONStorableFloat("Speed", 0f, -5f, 5f, true, false);
        
        public JSONStorableFloat sensitivity = new JSONStorableFloat("Sensitivity (ReadMyLips)", 1f, 0f, 5f);
        public JSONStorableFloat penetrationSoundsVolume = new JSONStorableFloat("Penetration Sounds Volume", 1f, 0f, 1f);
        
        public JSONStorableBool magnetic = new JSONStorableBool("Magnetic", true);
        
        protected FloatTriggerManager depthTriggers;
        protected FloatTriggerManager speedTriggers;
        
        protected UIDynamicButton depthTriggerButton;
        protected UIDynamicButton speedTriggerButton;
        
        public Transform enterPointTF;
        public CapsuleCollider proximityTrigger;
        public AudioSource audioSource;
        
        public Penetrator penetrator;
        
        public Force thrustForce;
        public Torque pitchTorque;
        public Torque rollTorque;
        public Torque yawTorque;
        public Force maleForce;

        private bool _thrustDirectionSet;
        private Vector3 _thrustDirection;
        public virtual Vector3 thrustDirection {
            get
            {
                if (_thrustDirectionSet || penetrator == null) return _thrustDirection;
                _thrustDirection = (penetrator.rootTransform.position - enterPointTF.position).normalized;
                _thrustDirectionSet = true;
                return _thrustDirection;
            }
        }
        public virtual Vector3 maleThrustDirection => -thrustDirection;
        
        protected Rigidbody rb;

        public Magnet magnet;
        
        // public MaleShaker maleShaker;

        public virtual void Init(string name)
        {
            this.name = name;
            depthTriggers = FillMeUp.singleton.AddFloatTriggerManager(depth, false, 0f, 0f, .25f);
            speedTriggers = FillMeUp.singleton.AddFloatTriggerManager(speed, true, 0f, 0f, .5f);
            if (this is Hand) rb = FillMeUp.atom.rigidbodies.First(x => x.name == name.First() + "Hand");
            else if(this is Throat) rb = FillMeUp.atom.rigidbodies.First(x => x.name == "head");
            else if(this is Cleavage) rb = FillMeUp.atom.rigidbodies.First(x => x.name == "chest");
            else rb = FillMeUp.atom.rigidbodies.First(x => x.name == "hip");
            thrustForce = gameObject.AddComponent<Force>().Init(name+":Thrust", rb, () => thrustDirection);
            thrustForce.enabledJ.setCallbackFunction += val =>
            {
                thrustForce.SetActive(val && enabled);
            };
            maleForce = gameObject.AddComponent<Force>().Init($"{name}:MaleThrust", null, () => maleThrustDirection);
            maleForce.paramControl.offset.val = 200f;
            maleForce.amplitude.mean.SetWithDefault(300f);
            maleForce.amplitude.delta.SetWithDefault(200f);
            maleForce.period.mean.SetWithDefault(.5f);
            maleForce.enabledJ.setCallbackFunction += val => maleForce.SetActive(val && enabled);
            
            FillMeUp.forceChooser.choices.Add(thrustForce.name);
            FillMeUp.forceChooser.choices.Add(maleForce.name);
            // CreateTriggersUI();
            InitForcePresets();
        }

        public virtual void FixedUpdate()
        {
            _thrustDirectionSet = false;
        }

        public virtual void ResetPenetration()
        {
        }

        public void InitForcePresets()
        {
            thrustForce.presetSystem.Init();
            maleForce.presetSystem.Init();
            if(pitchTorque) pitchTorque.presetSystem.Init();
            if(rollTorque) rollTorque.presetSystem.Init();
            if(yawTorque) yawTorque.presetSystem.Init();
        }

        public void ApplyLatestMatchingForcePresets()
        {
            thrustForce.presetSystem.ApplyLatestMatchingPreset();
            maleForce.presetSystem.ApplyLatestMatchingPreset();
            if(pitchTorque) pitchTorque.presetSystem.ApplyLatestMatchingPreset();
            if(rollTorque) rollTorque.presetSystem.ApplyLatestMatchingPreset();
            if(yawTorque) yawTorque.presetSystem.ApplyLatestMatchingPreset();
        }

        public virtual void SetPenetrator(CapsulePenetrator newPenetrator)
        {
            StimReceiver receiver;
            if (FillMeUp.stimReceivers.TryGetValue(newPenetrator, out receiver)) receiver.SetFucking(this);
        }

        public void ClearUI()
        {
            FillMeUp.singleton.RemoveUIElements(UIElements);
        }
        
        public virtual void SelectTab(int id)
        {
            FillMeUp.singleton.RemoveUIElements(UIElements);
            if(id < 4) FillMeUp.debugMode = false;
            if (id == 0) CreateTriggersUI();
            else if(id == 1) CreateSettingsUI();
            else if(id == 2) CreateBulgeUI();
            else if(id == 3) CreateForcesUI();
            else if(id == 4) CreateDebugUI();
        }
        
        public virtual JSONClass Store(string subScenePrefix, bool storeTriggers = true)
        {
            JSONClass jc = new JSONClass();
            magnetic.Store(jc);
            sensitivity.Store(jc);
            penetrationSoundsVolume.Store(jc);
            StoreForces(jc);
            // if (thrustForce) jc[thrustForce.name] = thrustForce.Store();
            // if (pitchTorque) jc[pitchTorque.name] = pitchTorque.Store();
            // if (rollTorque) jc[rollTorque.name] = rollTorque.Store();
            // if (yawTorque) jc[yawTorque.name] = yawTorque.Store();
            // if (maleForce) jc[maleForce.name] = maleForce.Store();
            if(storeTriggers)
            {
                jc["DepthTriggers"] = depthTriggers.Store();
                jc["SpeedTriggers"] = speedTriggers.Store();
            }
            return jc;
        }

        public void StoreForces(JSONClass jc)
        {
            if (thrustForce) jc[thrustForce.name] = thrustForce.Store();
            if (pitchTorque) jc[pitchTorque.name] = pitchTorque.Store();
            if (rollTorque) jc[rollTorque.name] = rollTorque.Store();
            if (yawTorque) jc[yawTorque.name] = yawTorque.Store();
            if (maleForce) jc[maleForce.name] = maleForce.Store();
        }
        
        public virtual void Load(JSONClass jc, string subScenePrefix)
        {
            if(jc.HasKey(name))
            {
                JSONClass tc = jc[name].AsObject;
                magnetic.Load(tc);
                sensitivity.Load(tc);
                penetrationSoundsVolume.Load(tc);
                LoadForces(tc);
                // if(thrustForce) thrustForce.Load(tc[thrustForce.name].AsObject);
                // if(pitchTorque) pitchTorque.Load(tc[pitchTorque.name].AsObject);
                // if(rollTorque) rollTorque.Load(tc[rollTorque.name].AsObject);
                // if(yawTorque) yawTorque.Load(tc[yawTorque.name].AsObject);
                // if(maleForce) maleForce.Load(tc[maleForce.name].AsObject);
                depthTriggers.Load(tc["DepthTriggers"].AsObject);
                speedTriggers.Load(tc["SpeedTriggers"].AsObject);
            }
            SyncTriggerButtons();
            // name.Print();
        }
        
        public void LoadForces(JSONClass jc)
        {
            if(thrustForce) thrustForce.Load(jc[thrustForce.name].AsObject);
            if(pitchTorque) pitchTorque.Load(jc[pitchTorque.name].AsObject);
            if(rollTorque) rollTorque.Load(jc[rollTorque.name].AsObject);
            if(yawTorque) yawTorque.Load(jc[yawTorque.name].AsObject);
            if(maleForce) maleForce.Load(jc[maleForce.name].AsObject);
        }
        
        public void StopForcesImmediate()
        {
            if(thrustForce) thrustForce.ShutDownImmediate();
            if(pitchTorque) pitchTorque.ShutDownImmediate();
            if(rollTorque) rollTorque.ShutDownImmediate();
            if(yawTorque) yawTorque.ShutDownImmediate();
            if(maleForce) maleForce.ShutDownImmediate();
        }
        
        public virtual void SyncTriggerButtons()
        {
            depthTriggerButton.label = $"Depth Triggers ({depthTriggers.triggers.Count})";
            speedTriggerButton.label = $"Speed Triggers ({speedTriggers.triggers.Count})";
        }

        public virtual void CreateTriggersUI()
        {
            depthTriggerButton = FillMeUp.singleton.CreateButton($"Depth Triggers ({depthTriggers.triggers.Count})");
            depthTriggerButton.buttonColor = new Color(0.45f, 1f, 0.45f);
            depthTriggerButton.button.onClick.AddListener(() =>
            {
                {
                    FillMeUp.singleton.ClearUI();
                    depthTriggers.OpenPanel(FillMeUp.singleton, FillMeUp.singleton.CreateUI);
                }
            });
            UIElements.Add(depthTriggerButton);
            depth.CreateUI(FillMeUp.singleton, UIElements:UIElements);
            
            speedTriggerButton = FillMeUp.singleton.CreateButton($"Speed Triggers ({speedTriggers.triggers.Count})", true);
            speedTriggerButton.buttonColor = new Color(0.45f, 1f, 0.45f);
            speedTriggerButton.button.onClick.AddListener(() =>
            {
                {
                    FillMeUp.singleton.ClearUI();
                    speedTriggers.OpenPanel(FillMeUp.singleton, FillMeUp.singleton.CreateUI);
                }
            });
            UIElements.Add(speedTriggerButton);
            speed.CreateUI(FillMeUp.singleton, UIElements:UIElements, rightSide:true);
            var textField = FillMeUp.singleton.CreateTextField(info, true);
            UIElements.Add(textField);
            textField.height = 50f;
        }

        public virtual void CreateSettingsUI()
        {
            
        }

        public virtual void CreateBulgeUI()
        {
            
        }

        protected static string toggleInfoText =
            "You can toggle all forces between 'off' and the state they were before by" +
            " pressing '<b>CTRL+F</b>' or calling the action '<b>Toggle Forces</b>' via trigger.\n" +
            "If you never want forces setup a FillMeUp preset called '<b>UserDefaults</b>'.";

        public virtual void CreateForcesUI()
        {
            thrustForce.enabledJ.CreateUI(UIElements);
            maleForce.enabledJ.CreateUI(UIElements);
            magnetic.CreateUI(UIElements);
            FillMeUp.singleton.SetupButton("Configure Thrust", true, () => CreateForceUINewPage(thrustForce), UIElements);
            FillMeUp.singleton.SetupButton("Configure Male Thrust", true, () => CreateForceUINewPage(maleForce), UIElements);
            if (magnet != null)
            {
                FillMeUp.singleton.SetupButton("Configure Magnet", true, magnet.CreateUINewPage, UIElements);
            }
            else
            {
                FillMeUp.singleton.SetupButton("N.A.", true, delegate {  }, UIElements);
            }
            if(this is Hand)
            {
                FillMeUp.singleton.SetupButton("Configure Hands Sync Group", true, FillMeUp.handForceGroup.CreateUI,
                    UIElements);
                FillMeUp.handForceGroup.driverInfo.CreateUI(UIElements);
            }
            else
            {
                FillMeUp.singleton.SetupButton("Configure Orifice Sync Group", true,
                    FillMeUp.orificeForceGroup.CreateUI, UIElements);
                FillMeUp.orificeForceGroup.driverInfo.CreateUI(UIElements);
            }

            var info = Utils.SetupInfoTextNoScroll(FillMeUp.singleton, toggleInfoText, 200f, false);
            info.background.offsetMin = new Vector2(0, 0);
            UIElements.Add(info);
        }

        public virtual void CreateForceUINewPage(Force force)
        {
            FillMeUp.singleton.ClearUI();
            UIElements.Clear();
            var button = FillMeUp.singleton.CreateButton("Return");
            button.buttonColor = new Color(0.55f, 0.90f, 1f);
            button.button.onClick.AddListener(
                () =>
                {
                    FillMeUp.singleton.ClearUI();
                    FillMeUp.singleton.CreateUI();
                    FillMeUp.singleton.settingsTabbar.SelectTab(3);
                    force.paramControl.UIOpen = false;
                });
            UIElements.Add(button);
            var chooser = FillMeUp.forceChooser.CreateUI(UIElements, true, chooserType:0);
            chooser.ForceHeight(50f);
            FillMeUp.forceChooser.valNoCallback = force.name;
            force.presetSystem.CreateUI();
            force.paramControl.CreateUI(FillMeUp.singleton);
            force.paramControl.UIOpen = true;
        }

        public virtual void CreateDebugUI()
        {
            FillMeUp.debugMode = true;
        }

        // public virtual void OnEnable()
        // {
        //     if(autoThrust.val) thrustForce.enabled = true;
        // }
    }
}