using UnityEngine;

namespace CheesyFX
{
    public class LimbForce
    {
        public bool enabled = true;
        private Rigidbody target;
        private Vector3 force;
        private Vector3 torque;

        private Vector3 forceTarget;
        private Vector3 torqueTarget;
        
        private float quickness;

        private JSONStorableFloat forceScale = new JSONStorableFloat("Force Scale", 50f, 0f, 200f);
        private JSONStorableFloat torqueScale = new JSONStorableFloat("Force Scale", 20f, 0f, 200f);
        private Vector3 directionaForcelScale = Vector3.one;
        private Vector3 directionaTorquelScale = Vector3.one;
        private float maxQuickness = .5f;

        public JSONStorableBool useForce = new JSONStorableBool("Use Force", true);
        public JSONStorableBool useTorque = new JSONStorableBool("Use Torque", true);

        private float timer;

        public LimbForce(Rigidbody rb, float forceScale = 1f, float torqueScale = 1f)
        {
            target = rb;
            // rb.transform.Draw();
            if (rb.name.Contains("Shldr"))
            {
                directionaTorquelScale = new Vector3(.1f, .5f, .5f);
            }
            else if (rb.name.Contains("head"))
            {
                this.torqueScale.val *= 3f;
            }
            else if (rb.name.Contains("Shin"))
            {
                useForce.val = false;
                directionaTorquelScale = new Vector3(10f, 2f, 1f);
            }
            else if (rb.name.Contains("Foot"))
            {
                directionaTorquelScale = new Vector3(3f, 1f, 1f);
                directionaForcelScale = new Vector3(3f, 1f, 1f);
            }
            else if (rb.name.Contains("Thigh"))
            {
                directionaForcelScale = new Vector3(3f, 3f, 5f);
                directionaTorquelScale = new Vector3(3f, 3f, 3f);
            }
            else if (rb.name.Contains("hip"))
            {
                directionaTorquelScale = new Vector3(3f, 1f, 3f);
            }
            // maxForceComponent *= forceScale;
            // maxTorqueComponent *= torqueScale;
        }

        public void Update()
        {
            if(!enabled) return;
            timer -= Time.fixedDeltaTime;
            if (timer < 0f)
            {
                timer = Random.Range(1f, 10f);
                quickness = Random.Range(.1f, maxQuickness);
                if(useForce.val)
                {
                    // forceTarget = new Vector3(
                    //     Random.Range(-maxForceComponent, maxForceComponent),
                    //     Random.Range(-maxForceComponent, maxForceComponent),
                    //     Random.Range(-maxForceComponent, maxForceComponent));
                    var direction = Random.onUnitSphere;
                    direction.Scale(directionaForcelScale);
                    forceTarget = NormalDistribution.GetValue(0f, forceScale.val, onesided:true, sharpness:2) * direction;
                }
                if(useTorque.val)
                {
                    // torqueTarget = new Vector3(
                    //     2f * Random.Range(-maxTorqueComponent, maxTorqueComponent),
                    //     Random.Range(-maxTorqueComponent, maxTorqueComponent),
                    //     Random.Range(-maxTorqueComponent, maxTorqueComponent));
                    var direction = Random.onUnitSphere;
                    direction.Scale(directionaTorquelScale);
                    torqueTarget = NormalDistribution.GetValue(0f, torqueScale.val, onesided:true, sharpness:2) * direction;
                }
                
            }
            if(useForce.val)
            {
                force = Vector3.Lerp(force, forceTarget, quickness * Time.fixedDeltaTime);
                torque = Vector3.Lerp(torque, torqueTarget, quickness * Time.fixedDeltaTime);
            }
            if(useTorque.val)
            {
                target.AddForce(target.transform.TransformDirection(force));
                target.AddTorque(target.transform.TransformDirection(torque));
            }
        }
    }
}