using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace CheesyFX
{
    public class GenderListener
    {
        public Atom atom;
        public DAZCharacterSelector dcs;
        private DAZCharacterSelector.Gender gender;
        public bool isFuta;
        private UnityEventsListener eventListener;
        public readonly UnityEvent OnGenderChanged = new UnityEvent();
        public readonly UnityEvent OnChangedToFemale = new UnityEvent();
        public readonly UnityEvent OnChangedToMale = new UnityEvent();
        public readonly UnityEvent OnChangedToFuta = new UnityEvent();

        public GenderListener(Atom atom)
        {
            this.atom = atom;
            dcs = atom.GetStorableByID("geometry") as DAZCharacterSelector;
            gender = dcs.gender;
            var gen1Object = atom.GetStorableByID("PhysicsModel").transform.GetAllChildren()
                .FirstOrDefault(x => x.name == "Gen1").gameObject;
            isFuta = gen1Object.activeSelf && dcs.gender == DAZCharacterSelector.Gender.Female;
            eventListener = gen1Object.AddComponent<UnityEventsListener>();
            eventListener.onEnabled.AddListener(OnGen1Enabled);
            eventListener.onDisabled.AddListener(OnGen1Disabled);
        }

        private void OnGen1Enabled()
        {
            if (dcs.gender == DAZCharacterSelector.Gender.Female)
            {
                OnChangedToFuta.Invoke();
                isFuta = true;
            }
            else
            {
                OnChangedToMale.Invoke();
                isFuta = false;
            }
            if(dcs.gender != gender) OnGenderChanged.Invoke();
            gender = dcs.gender;
        }

        public void OnGen1Disabled()
        {
            // "OnGen1Disabled".Print();
            // dcs.gender.Print();
            // dcs.containingAtom.Print();
            if (dcs.gender == DAZCharacterSelector.Gender.Male) return;
            OnChangedToFemale.Invoke();
            if(dcs.gender != gender) OnGenderChanged.Invoke();
            gender = dcs.gender;
            isFuta = false;
        }

        public void Destroy()
        {
            Object.Destroy(eventListener);
        }
    }
}